@@ -1,28 +1,35 @@
-/// Code based on https://github.com/defuz/sublimate/blob/master/src/core/syntax/scope.rs
+/// Code based on <https://github.com/defuz/sublimate/blob/master/src/core/syntax/scope.rs>
 /// released under the MIT license by @defuz
 use crate::parsing::{Scope, ScopeStack, MatchPower, ParseScopeError};
 use std::str::FromStr;
 
-/// A single selector consisting of a stack to match and a possible stack to exclude from being matched.
-/// You probably want `ScopeSelectors` which is this but with union support.
+/// A single selector consisting of a stack to match and a possible stack to
+/// exclude from being matched.
+///
+/// You probably want [`ScopeSelectors`] which is this but with union support.
+///
+/// [`ScopeSelectors`]: struct.ScopeSelectors.html
 #[derive(Debug, Clone, PartialEq, Eq, Default, Serialize, Deserialize)]
 pub struct ScopeSelector {
-    path: ScopeStack,
-    excludes: Vec<ScopeStack>,
+    pub path: ScopeStack,
+    pub excludes: Vec<ScopeStack>,
 }
 
 /// A selector set that matches anything matched by any of its component selectors.
-/// See [The TextMate Docs](https://manual.macromates.com/en/scope_selectors) for how these
-/// work.
+///
+/// See [The TextMate Docs](https://manual.macromates.com/en/scope_selectors) for how these work.
 #[derive(Debug, Clone, PartialEq, Eq, Default, Serialize, Deserialize)]
 pub struct ScopeSelectors {
-    /// the selectors, if any of them match, this matches
+    /// The selectors, if any of them match, that this matches
     pub selectors: Vec<ScopeSelector>,
 }
 
 impl ScopeSelector {
     /// Checks if this selector matches a given scope stack.
-    /// See `ScopeSelectors#does_match` for more info.
+    ///
+    /// See [`ScopeSelectors::does_match`] for more info.
+    ///
+    /// [`ScopeSelectors::does_match`]: struct.ScopeSelectors.html#method.does_match
     pub fn does_match(&self, stack: &[Scope]) -> Option<MatchPower> {
         // if there are any exclusions, and any one of them matches, then this selector doesn't match
         if self.excludes.iter().any(|sel| sel.is_empty() || sel.does_match(stack).is_some()) {
@@ -43,6 +50,11 @@ impl ScopeSelector {
         }
         Some(self.path.as_slice()[0])
     }
+
+    /// Extract all selectors for generating CSS
+    pub fn extract_scopes(&self) -> Vec<Scope> {
+        self.path.scopes.clone()
+    }
 }
 
 impl FromStr for ScopeSelector {
@@ -67,10 +79,10 @@ impl FromStr for ScopeSelector {
 }
 
 impl ScopeSelectors {
-    /// checks if any of these selectors match the given scope stack
-    /// if so it returns a match score, higher match scores are stronger
-    /// matches. Scores are ordered according to the rules found
-    /// at https://manual.macromates.com/en/scope_selectors
+    /// Checks if any of the given selectors match the given scope stack
+    ///
+    /// If so, it returns a match score. Higher match scores indicate stronger matches. Scores are
+    /// ordered according to the rules found at [https://manual.macromates.com/en/scope_selectors](https://manual.macromates.com/en/scope_selectors).
     ///
     /// # Examples
     ///
@@ -113,7 +125,7 @@ mod tests {
         let first_sel = &sels.selectors[0];
         assert_eq!(format!("{:?}", first_sel),
                    "ScopeSelector { path: ScopeStack { clear_stack: [], scopes: [<source.php>, <meta.preprocessor>] }, excludes: [ScopeStack { clear_stack: [], scopes: [<string.quoted>] }] }");
-        
+
         let sels = ScopeSelectors::from_str("source.php meta.preprocessor -string.quoted|\
                                              source string")
             .unwrap();
@@ -121,21 +133,21 @@ mod tests {
         let first_sel = &sels.selectors[0];
         assert_eq!(format!("{:?}", first_sel),
                    "ScopeSelector { path: ScopeStack { clear_stack: [], scopes: [<source.php>, <meta.preprocessor>] }, excludes: [ScopeStack { clear_stack: [], scopes: [<string.quoted>] }] }");
-        
+
         let sels = ScopeSelectors::from_str("text.xml meta.tag.preprocessor.xml punctuation.separator.key-value.xml")
             .unwrap();
         assert_eq!(sels.selectors.len(), 1);
         let first_sel = &sels.selectors[0];
         assert_eq!(format!("{:?}", first_sel),
                    "ScopeSelector { path: ScopeStack { clear_stack: [], scopes: [<text.xml>, <meta.tag.preprocessor.xml>, <punctuation.separator.key-value.xml>] }, excludes: [] }");
-        
+
         let sels = ScopeSelectors::from_str("text.xml meta.tag.preprocessor.xml punctuation.separator.key-value.xml - text.html - string")
             .unwrap();
         assert_eq!(sels.selectors.len(), 1);
         let first_sel = &sels.selectors[0];
         assert_eq!(format!("{:?}", first_sel),
                    "ScopeSelector { path: ScopeStack { clear_stack: [], scopes: [<text.xml>, <meta.tag.preprocessor.xml>, <punctuation.separator.key-value.xml>] }, excludes: [ScopeStack { clear_stack: [], scopes: [<text.html>] }, ScopeStack { clear_stack: [], scopes: [<string>] }] }");
-        
+
         let sels = ScopeSelectors::from_str("text.xml meta.tag.preprocessor.xml punctuation.separator.key-value.xml - text.html - string, source - comment")
             .unwrap();
         assert_eq!(sels.selectors.len(), 2);
@@ -145,7 +157,7 @@ mod tests {
         let second_sel = &sels.selectors[1];
         assert_eq!(format!("{:?}", second_sel),
                    "ScopeSelector { path: ScopeStack { clear_stack: [], scopes: [<source>] }, excludes: [ScopeStack { clear_stack: [], scopes: [<comment>] }] }");
-        
+
         let sels = ScopeSelectors::from_str(" -a.b|j.g")
             .unwrap();
         assert_eq!(sels.selectors.len(), 2);
@@ -185,7 +197,7 @@ mod tests {
                        .does_match(ScopeStack::from_str("a.b c.d e.f").unwrap().as_slice()),
                    Some(MatchPower(0o201u64 as f64)));
     }
-    
+
     #[test]
     fn empty_stack_matching_works() {
         use crate::parsing::{ScopeStack, MatchPower};
@@ -226,7 +238,7 @@ mod tests {
                        .unwrap()
                        .does_match(ScopeStack::from_str("a.b c.d j e.f").unwrap().as_slice()),
                    Some(MatchPower(0o1u64 as f64)));
-        
+
         assert_eq!(ScopeSelector::from_str(" -a.b")
                        .unwrap()
                        .does_match(ScopeStack::from_str("a.b c.d j e.f").unwrap().as_slice()),
@@ -260,7 +272,7 @@ mod tests {
                        .does_match(ScopeStack::from_str("a.b c.d j e.f").unwrap().as_slice()),
                    Some(MatchPower(0o1u64 as f64)));
     }
-    
+
     #[test]
     fn multiple_excludes_matching_works() {
         use crate::parsing::{ScopeStack, MatchPower};
