@@ -1,7 +1,7 @@
-/// Code based on https://github.com/defuz/sublimate/blob/master/src/core/settings.rs
+/// Code based on <https://github.com/defuz/sublimate/blob/master/src/core/settings.rs>
 /// released under the MIT license by @defuz
+use plist::Error as PlistError;
 use std::io::{Read, Seek};
-use plist::{Error as PlistError};
 
 pub use serde_json::Value as Settings;
 pub use serde_json::Value::Array as SettingsArray;
