@@ -3,17 +3,23 @@
 //! Much more info about syntect is available on the [Github Page](https://github.com/trishume/syntect).
 //!
 //! May I suggest that you start by reading the `Readme.md` file in the main repo.
-//! Once you're done with that you can look at the docs for `parsing::SyntaxSet`
-//! and for the `easy` module.
+//! Once you're done with that you can look at the docs for [`parsing::SyntaxSet`]
+//! and for the [`easy`] module.
 //!
-//! Almost everything in syntect is divided up into either the `parsing` module
-//! for turning text into text annotated with scopes, and the `highlighting` module
+//! Almost everything in syntect is divided up into either the [`parsing`] module
+//! for turning text into text annotated with scopes, and the [`highlighting`] module
 //! for turning annotated text into styled/colored text.
 //!
-//! Some docs have example code but a good place to look is the `syncat` example as well as the source code
-//! for the `easy` module in `easy.rs` as that shows how to plug the various parts together for common use cases.
+//! Some docs have example code but a good place to look is the `syncat` example as
+//! well as the source code for the [`easy`] module in `easy.rs` as that shows how to
+//! plug the various parts together for common use cases.
+//!
+//! [`parsing::SyntaxSet`]: parsing/struct.SyntaxSet.html
+//! [`easy`]: easy/index.html
+//! [`parsing`]: parsing/index.html
+//! [`highlighting`]: highlighting/index.html
 
-#![doc(html_root_url = "https://docs.rs/syntect/3.3.0")]
+#![doc(html_root_url = "https://docs.rs/syntect/4.6.0")]
 
 #[macro_use]
 extern crate lazy_static;
@@ -109,33 +115,21 @@ impl fmt::Display for LoadingError {
             #[cfg(feature = "yaml-load")]
             ParseSyntax(ref error, ref filename) => {
                 if let Some(ref file) = filename {
-                    write!(f, "{}: {}", file, error.description())
+                    write!(f, "{}: {}", file, error)
                 } else {
                     error.fmt(f)
                 }
             },
-            _ => write!(f, "{}", self.description()),
+            #[cfg(feature = "metadata")]
+            ParseMetadata(_) => write!(f, "Failed to parse JSON"),
+            ParseTheme(_) => write!(f, "Invalid syntax theme"),
+            ReadSettings(_) => write!(f, "Invalid syntax theme settings"),
+            BadPath => write!(f, "Invalid path"),
         }
     }
 }
 
 impl Error for LoadingError {
-    fn description(&self) -> &str {
-        use crate::LoadingError::*;
-
-        match *self {
-            WalkDir(ref error) => error.description(),
-            Io(ref error) => error.description(),
-            #[cfg(feature = "yaml-load")]
-            ParseSyntax(ref error, ..) => error.description(),
-            #[cfg(feature = "metadata")]
-            ParseMetadata(_) => "Failed to parse JSON",
-            ParseTheme(_) => "Invalid syntax theme",
-            ReadSettings(_) => "Invalid syntax theme settings",
-            BadPath => "Invalid path",
-        }
-    }
-
     fn cause(&self) -> Option<&dyn Error> {
         use crate::LoadingError::*;
 
