@@ -1,6 +1,5 @@
-//! Support for loading `.tmPreferences` metadata files. These files contain
-//! information related to indentation rules, comment markers,
-//! and other syntax-specific things.
+//! Support for loading `.tmPreferences` metadata files, with information related to indentation
+//! rules, comment markers, and other syntax-specific things.
 
 use std::collections::BTreeMap;
 use std::path::PathBuf;
@@ -8,11 +7,10 @@ use std::fs::File;
 use std::io::BufReader;
 use std::str::FromStr;
 
-use lazycell::AtomicLazyCell;
-use onig::{Regex, SearchOptions};
 use serde::{Deserialize, Deserializer, Serialize, Serializer};
 use serde_json;
 
+use super::regex::Regex;
 use super::scope::{MatchPower, Scope};
 use super::super::LoadingError;
 use super::super::highlighting::settings::*;
@@ -20,23 +18,20 @@ use super::super::highlighting::ScopeSelectors;
 
 type Dict = serde_json::Map<String, Settings>;
 
-/// A String representation of a `ScopeSelectors` instance.
+/// A String representation of a [`ScopeSelectors`] instance.
+///
+/// [`ScopeSelectors`]: ../../highlighting/struct.ScopeSelectors.html
 type SelectorString = String;
 
-/// A simple regex pattern, used for checking indentation state.
-#[derive(Debug)]
-pub struct Pattern {
-    pub regex_str: String,
-    pub regex: AtomicLazyCell<Regex>,
-}
-
-/// A collection of all loaded metadata.
+/// A collection of all loaded metadata
 #[derive(Debug, Default, Clone, Serialize, Deserialize)]
 pub struct Metadata {
     pub scoped_metadata: Vec<MetadataSet>,
 }
 
-/// Metadata for a particular `ScopeSelector`.
+/// Metadata for a particular [`ScopeSelector`]
+///
+/// [`ScopeSelector`]: ../../highlighting/struct.ScopeSelector.html
 #[derive(Debug, Clone, PartialEq)]
 pub struct MetadataSet {
     /// The raw string representation of this selector. We keep this around
@@ -54,11 +49,11 @@ pub struct MetadataSet {
 #[derive(Debug, Clone, Serialize, Deserialize, PartialEq)]
 #[serde(rename_all = "camelCase")]
 pub struct MetadataItems {
-    pub increase_indent_pattern: Option<Pattern>,
-    pub decrease_indent_pattern: Option<Pattern>,
-    pub bracket_indent_next_line_pattern: Option<Pattern>,
-    pub disable_indent_next_line_pattern: Option<Pattern>,
-    pub unindented_line_pattern: Option<Pattern>,
+    pub increase_indent_pattern: Option<Regex>,
+    pub decrease_indent_pattern: Option<Regex>,
+    pub bracket_indent_next_line_pattern: Option<Regex>,
+    pub disable_indent_next_line_pattern: Option<Regex>,
+    pub unindented_line_pattern: Option<Regex>,
     pub indent_parens: Option<bool>,
     #[serde(default)]
     pub shell_variables: BTreeMap<String, String>,
@@ -102,14 +97,20 @@ const KEYS_WE_USE: &[&str] = &[
 ];
 
 impl LoadMetadata {
-    /// Adds the provided `RawMetadataEntry`. When creating the final `Metadata`
-    /// object, all `RawMetadataEntry` items are sorted by path, and items that
-    /// share a scope selector are merged; last writer wins.
+    /// Adds the provided `RawMetadataEntry`
+    ///
+    /// When creating the final [`Metadata`] object, all [`RawMetadataEntry`] items are sorted by
+    /// path, and items that share a scope selector are merged; last writer wins.
+    ///
+    /// [`Metadata`]: struct.Metadata.html
+    /// [`RawMetadataEntry`]: struct.RawMetadataEntry.html
     pub fn add_raw(&mut self, raw: RawMetadataEntry) {
         self.loaded.push(raw);
     }
 
-    /// Generates a `MetadataSet` from a single file
+    /// Generates a [`MetadataSet`] from a single file
+    ///
+    /// [`MetadataSet`]: struct.MetadataSet.html
     #[cfg(test)]
     pub(crate) fn quick_load(path: &str) -> Result<MetadataSet, LoadingError> {
         let mut loaded = Self::default();
@@ -179,8 +180,10 @@ fn append_vars(obj: &mut Dict, vars: Settings, scope: &str) {
 }
 
 impl Metadata {
-    /// For a given stack of scopes, returns a [`ScopedMetadata`] object
-    /// which provides convenient access to metadata items which match the stack.
+    /// For a given stack of scopes, returns a [`ScopedMetadata`] object which provides convenient
+    /// access to metadata items which match the stack.
+    ///
+    /// [`ScopedMetadata`]: struct.ScopedMetadata.html
     pub fn metadata_for_scope(&self, scope: &[Scope]) -> ScopedMetadata<'_> {
         let mut metadata_matches = self.scoped_metadata
             .iter()
@@ -264,8 +267,8 @@ impl MetadataSet {
     }
 }
 
-/// A collection of `MetadataSet`s which match a given scope selector,
-/// sorted in order of the strength of the match.
+/// A collection of [`MetadataSet`]s which match a given scope selector, sorted by the strength of
+/// the match
 ///
 /// # Examples
 ///
@@ -307,6 +310,8 @@ impl MetadataSet {
 /// // and the other match is used when it does not.
 /// assert!(scoped.decrease_indent("one decrease"));
 /// ```
+///
+/// [`MetadataSet`]: struct.MetadataSet.html
 #[derive(Debug, Clone)]
 pub struct ScopedMetadata<'a> {
     pub items: Vec<(MatchPower, &'a MetadataSet)>,
@@ -377,56 +382,6 @@ impl RawMetadataEntry {
     }
 }
 
-impl Pattern {
-    pub fn is_match<S: AsRef<str>>(&self, string: S) -> bool {
-        self.regex()
-            .match_with_options(
-                string.as_ref(),
-                0,
-                SearchOptions::SEARCH_OPTION_NONE,
-                None)
-            .is_some()
-    }
-
-    pub fn regex(&self) -> &Regex {
-        if let Some(regex) = self.regex.borrow() {
-            regex
-        } else {
-            let regex = Regex::new(&self.regex_str)
-                .expect("regex string should be pre-tested");
-            self.regex.fill(regex).ok();
-            self.regex.borrow().unwrap()
-        }
-    }
-}
-
-impl Clone for Pattern {
-    fn clone(&self) -> Self {
-        Pattern { regex_str: self.regex_str.clone(), regex: AtomicLazyCell::new() }
-    }
-}
-
-impl PartialEq for Pattern {
-    fn eq(&self, other: &Pattern) -> bool {
-        self.regex_str == other.regex_str
-    }
-}
-
-impl Eq for Pattern {}
-
-impl Serialize for Pattern {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error> where S: Serializer {
-        serializer.serialize_str(&self.regex_str)
-    }
-}
-
-impl<'de> Deserialize<'de> for Pattern {
-    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error> where D: Deserializer<'de> {
-        let regex_str = String::deserialize(deserializer)?;
-        Ok(Pattern { regex_str, regex: AtomicLazyCell::new() })
-    }
-}
-
 #[derive(Serialize, Deserialize)]
 struct MetaSetSerializable {
     selector_string: String,
@@ -525,14 +480,6 @@ mod tests {
         assert!(metadata.items.increase_indent_pattern.is_none());
     }
 
-    #[test]
-    fn serde_pattern() {
-        let pattern: Pattern = serde_json::from_str("\"just a string\"").unwrap();
-        assert_eq!(pattern.regex_str, "just a string");
-        let back_to_str = serde_json::to_string(&pattern).unwrap();
-        assert_eq!(back_to_str, "\"just a string\"");
-    }
-
     #[test]
     fn indent_rust() {
         let ps = SyntaxSet::load_from_folder("testdata/Packages/Rust").unwrap();
