@@ -7,7 +7,7 @@ use super::metadata::{LoadMetadata, Metadata, RawMetadataEntry};
 #[cfg(feature = "yaml-load")]
 use super::super::LoadingError;
 
-use std::collections::{HashMap, HashSet};
+use std::collections::{HashMap, HashSet, BTreeSet};
 use std::path::Path;
 #[cfg(feature = "yaml-load")]
 use walkdir::WalkDir;
@@ -18,15 +18,18 @@ use std::fs::File;
 use std::mem;
 
 use lazycell::AtomicLazyCell;
-use onig::Regex;
+use super::regex::Regex;
 use crate::parsing::syntax_definition::ContextId;
 
 /// A syntax set holds multiple syntaxes that have been linked together.
 ///
-/// Use a `SyntaxSetBuilder` to load syntax definitions and build a syntax set.
+/// Use a [`SyntaxSetBuilder`] to load syntax definitions and build a syntax set.
 ///
-/// After building, the syntax set is immutable and can no longer be modified.
-/// But you can convert it back to a builder by using `into_builder`.
+/// After building, the syntax set is immutable and can no longer be modified, but you can convert
+/// it back into a builder by using the [`into_builder`] method.
+///
+/// [`SyntaxSetBuilder`]: struct.SyntaxSetBuilder.html
+/// [`into_builder`]: #method.into_builder
 #[derive(Debug, Serialize, Deserialize)]
 pub struct SyntaxSet {
     syntaxes: Vec<SyntaxReference>,
@@ -37,14 +40,17 @@ pub struct SyntaxSet {
     #[serde(skip_serializing, skip_deserializing, default = "AtomicLazyCell::new")]
     first_line_cache: AtomicLazyCell<FirstLineCache>,
     /// Metadata, e.g. indent and commenting information.
-    /// NOTE: if serializing, you should handle metadata manually; that is,
-    /// you should serialize and deserialize it separately. See
-    /// `examples/gendata.rs` for an example.
+    ///
+    /// NOTE: if serializing, you should handle metadata manually; that is, you should serialize and
+    /// deserialize it separately. See `examples/gendata.rs` for an example.
     #[cfg(feature = "metadata")]
     #[serde(skip, default)]
     pub(crate) metadata: Metadata,
 }
 
+/// A linked version of a [`SyntaxDefinition`] that is only useful as part of the
+/// [`SyntaxSet`] that contains it. See docs for [`SyntaxSetBuilder::build`] for
+/// more info.
 #[derive(Clone, Debug, Serialize, Deserialize)]
 pub struct SyntaxReference {
     pub name: String,
@@ -59,16 +65,21 @@ pub struct SyntaxReference {
 }
 
 /// A syntax set builder is used for loading syntax definitions from the file
-/// system or by adding `SyntaxDefinition` objects.
+/// system or by adding [`SyntaxDefinition`] objects.
 ///
-/// Once all the syntaxes have been added, call `build` to turn the builder into
-/// a `SyntaxSet` that can be used for parsing or highlighting.
+/// Once all the syntaxes have been added, call [`build`] to turn the builder into
+/// a [`SyntaxSet`] that can be used for parsing or highlighting.
+///
+/// [`SyntaxDefinition`]: syntax_definition/struct.SyntaxDefinition.html
+/// [`build`]: #method.build
+/// [`SyntaxSet`]: struct.SyntaxSet.html
 #[derive(Clone, Default)]
 pub struct SyntaxSetBuilder {
     syntaxes: Vec<SyntaxDefinition>,
     path_syntaxes: Vec<(String, usize)>,
     #[cfg(feature = "metadata")]
     raw_metadata: LoadMetadata,
+
     /// If this `SyntaxSetBuilder` is created with `SyntaxSet::into_builder`
     /// from a `SyntaxSet` that already had metadata, we keep that metadata,
     /// merging it with newly loaded metadata.
@@ -130,8 +141,11 @@ impl SyntaxSet {
     /// definitions from a folder and then building the syntax set.
     ///
     /// Note that this uses `lines_include_newline` set to `false`, see the
-    /// `add_from_folder` method docs on `SyntaxSetBuilder` for an explanation
+    /// [`add_from_folder`] method docs on [`SyntaxSetBuilder`] for an explanation
     /// as to why this might not be the best.
+    ///
+    /// [`add_from_folder`]: struct.SyntaxSetBuilder.html#method.add_from_folder
+    /// [`SyntaxSetBuilder`]: struct.SyntaxSetBuilder.html
     #[cfg(feature = "yaml-load")]
     pub fn load_from_folder<P: AsRef<Path>>(folder: P) -> Result<SyntaxSet, LoadingError> {
         let mut builder = SyntaxSetBuilder::new();
@@ -148,7 +162,7 @@ impl SyntaxSet {
     pub fn set_metadata(&mut self, metadata: Metadata) {
         self.metadata = metadata;
     }
-    
+
     /// The loaded metadata for this set.
     #[cfg(feature = "metadata")]
     pub fn metadata(&self) -> &Metadata {
@@ -156,8 +170,10 @@ impl SyntaxSet {
     }
 
     /// Finds a syntax by its default scope, for example `source.regexp` finds the regex syntax.
+    ///
     /// This and all similar methods below do a linear search of syntaxes, this should be fast
-    /// because there aren't many syntaxes, but don't think you can call it a bajillion times per second.
+    /// because there aren't many syntaxes, but don't think you can call it a bajillion times per
+    /// second.
     pub fn find_syntax_by_scope(&self, scope: Scope) -> Option<&SyntaxReference> {
         self.syntaxes.iter().rev().find(|&s| s.scope == scope)
     }
@@ -171,8 +187,9 @@ impl SyntaxSet {
     }
 
     /// Searches for a syntax first by extension and then by case-insensitive name
-    /// useful for things like Github-flavoured-markdown code block highlighting where
-    /// all you have to go on is a short token given by the user
+    ///
+    /// This is useful for things like Github-flavoured-markdown code block highlighting where all
+    /// you have to go on is a short token given by the user
     pub fn find_syntax_by_token<'a>(&'a self, s: &str) -> Option<&'a SyntaxReference> {
         {
             let ext_res = self.find_syntax_by_extension(s);
@@ -183,13 +200,14 @@ impl SyntaxSet {
         self.syntaxes.iter().rev().find(|&syntax| syntax.name.eq_ignore_ascii_case(s))
     }
 
-    /// Try to find the syntax for a file based on its first line.
-    /// This uses regexes that come with some sublime syntax grammars
-    /// for matching things like shebangs and mode lines like `-*- Mode: C -*-`
+    /// Try to find the syntax for a file based on its first line
+    ///
+    /// This uses regexes that come with some sublime syntax grammars for matching things like
+    /// shebangs and mode lines like `-*- Mode: C -*-`
     pub fn find_syntax_by_first_line<'a>(&'a self, s: &str) -> Option<&'a SyntaxReference> {
         let cache = self.first_line_cache();
         for &(ref reg, i) in cache.regexes.iter().rev() {
-            if reg.find(s).is_some() {
+            if reg.search(s, 0, s.len(), None) {
                 return Some(&self.syntaxes[i]);
             }
         }
@@ -197,10 +215,11 @@ impl SyntaxSet {
     }
 
     /// Searches for a syntax by it's original file path when it was first loaded from disk
-    /// primarily useful for syntax tests
-    /// some may specify a Packages/PackageName/SyntaxName.sublime-syntax path
-    /// others may just have SyntaxName.sublime-syntax
-    /// this caters for these by matching the end of the path of the loaded syntax definition files
+    ///
+    /// This is primarily useful for syntax tests. Some may specify a
+    /// `Packages/PackageName/SyntaxName.sublime-syntax` path, and others may just have
+    /// `SyntaxName.sublime-syntax`. This caters for these by matching the end of the path of the
+    /// loaded syntax definition files
     // however, if a syntax name is provided without a folder, make sure we don't accidentally match the end of a different syntax definition's name - by checking a / comes before it or it is the full path
     pub fn find_syntax_by_path<'a>(&'a self, path: &str) -> Option<&'a SyntaxReference> {
         let mut slash_path = "/".to_string();
@@ -208,12 +227,15 @@ impl SyntaxSet {
         self.path_syntaxes.iter().rev().find(|t| t.0.ends_with(&slash_path) || t.0 == path).map(|&(_,i)| &self.syntaxes[i])
     }
 
-    /// Convenience method that tries to find the syntax for a file path,
-    /// first by extension/name and then by first line of the file if that doesn't work.
+    /// Convenience method that tries to find the syntax for a file path, first by extension/name
+    /// and then by first line of the file if that doesn't work.
+    ///
     /// May IO Error because it sometimes tries to read the first line of the file.
     ///
     /// # Examples
-    /// When determining how to highlight a file, use this in combination with a fallback to plain text:
+    ///
+    /// When determining how to highlight a file, use this in combination with a fallback to plain
+    /// text:
     ///
     /// ```
     /// use syntect::parsing::SyntaxSet;
@@ -245,11 +267,12 @@ impl SyntaxSet {
     }
 
     /// Finds a syntax for plain text, which usually has no highlighting rules.
-    /// Good as a fallback when you can't find another syntax but you still want
-    /// to use the same highlighting pipeline code.
     ///
-    /// This syntax should always be present, if not this method will panic.
-    /// If the way you load syntaxes doesn't create one, use `add_plain_text_syntax`.
+    /// This is good as a fallback when you can't find another syntax but you still want to use the
+    /// same highlighting pipeline code.
+    ///
+    /// This syntax should always be present, if not this method will panic. If the way you load
+    /// syntaxes doesn't create one, use [`add_plain_text_syntax`].
     ///
     /// # Examples
     /// ```
@@ -260,6 +283,8 @@ impl SyntaxSet {
     /// let syntax = ss.find_syntax_by_token("rs").unwrap_or_else(|| ss.find_syntax_plain_text());
     /// assert_eq!(syntax.name, "Plain Text");
     /// ```
+    ///
+    /// [`add_plain_text_syntax`]: struct.SyntaxSetBuilder.html#method.add_plain_text_syntax
     pub fn find_syntax_plain_text(&self) -> &SyntaxReference {
         self.find_syntax_by_name("Plain Text")
             .expect("All syntax sets ought to have a plain text syntax")
@@ -337,6 +362,62 @@ impl SyntaxSet {
             self.first_line_cache.borrow().unwrap()
         }
     }
+
+    pub fn find_unlinked_contexts(&self) -> BTreeSet<String> {
+        let SyntaxSet { syntaxes, contexts, .. } = self;
+
+        let mut context_map = HashMap::with_capacity(contexts.len());
+        for (i, context) in contexts.into_iter().enumerate() {
+            context_map.insert(i, context);
+        }
+
+        let mut unlinked_contexts = BTreeSet::new();
+
+        for syntax in syntaxes {
+            let SyntaxReference {
+                name,
+                scope,
+                contexts,
+                ..
+            } = syntax;
+
+            for (_, context_id) in contexts {
+                if let Some(context) = context_map.remove(&context_id.index()) {
+                    for pattern in context.patterns.iter() {
+                        let maybe_refs_to_check = match pattern {
+                            Pattern::Match(match_pat) => {
+                                match &match_pat.operation {
+                                    MatchOperation::Push(context_refs) => {
+                                        Some(context_refs)
+                                    },
+                                    MatchOperation::Set(context_refs) => {
+                                        Some(context_refs)
+                                    },
+                                    _ => None,
+                                }
+                            },
+                            _ => None,
+                        };
+
+                        for context_ref in maybe_refs_to_check.into_iter().flatten() {
+                            match context_ref {
+                                ContextReference::Direct(_) => {},
+                                _ => {
+                                    unlinked_contexts.insert(
+                                        format!(
+                                            "Syntax '{}' with scope '{}' has unresolved context reference {:?}",
+                                            name, scope, &context_ref
+                                        )
+                                    );
+                                },
+                            }
+                        }
+                    }
+                }
+            }
+        }
+        unlinked_contexts
+    }
 }
 
 
@@ -350,9 +431,15 @@ impl SyntaxSetBuilder {
         self.syntaxes.push(syntax);
     }
 
-    /// Rarely useful method that loads in a syntax with no highlighting rules for plain text.
-    /// Exists mainly for adding the plain text syntax to syntax set dumps, because for some
-    /// reason the default Sublime plain text syntax is still in `.tmLanguage` format.
+    /// The list of syntaxes added so far.
+    pub fn syntaxes(&self) -> &[SyntaxDefinition] {
+        &self.syntaxes[..]
+    }
+
+    /// A rarely useful method that loads in a syntax with no highlighting rules for plain text
+    ///
+    /// Exists mainly for adding the plain text syntax to syntax set dumps, because for some reason
+    /// the default Sublime plain text syntax is still in `.tmLanguage` format.
     #[cfg(feature = "yaml-load")]
     pub fn add_plain_text_syntax(&mut self) {
         let s = "---\nname: Plain Text\nfile_extensions: [txt]\nscope: text.plain\ncontexts: \
@@ -361,17 +448,19 @@ impl SyntaxSetBuilder {
         self.syntaxes.push(syn);
     }
 
-    /// Loads all the .sublime-syntax files in a folder into this builder.
+    /// Loads all the `.sublime-syntax` files in a folder into this builder.
     ///
-    /// The `lines_include_newline` parameter is used to work around the fact that Sublime Text normally
-    /// passes line strings including newline characters (`\n`) to its regex engine. This results in many
-    /// syntaxes having regexes matching `\n`, which doesn't work if you don't pass in newlines.
-    /// It is recommended that if you can you pass in lines with newlines if you can and pass `true` for this parameter.
-    /// If that is inconvenient pass `false` and the loader will do some hacky find and replaces on the
-    /// match regexes that seem to work for the default syntax set, but may not work for any other syntaxes.
+    /// The `lines_include_newline` parameter is used to work around the fact that Sublime Text
+    /// normally passes line strings including newline characters (`\n`) to its regex engine. This
+    /// results in many syntaxes having regexes matching `\n`, which doesn't work if you don't pass
+    /// in newlines. It is recommended that if you can you pass in lines with newlines if you can
+    /// and pass `true` for this parameter. If that is inconvenient pass `false` and the loader
+    /// will do some hacky find and replaces on the match regexes that seem to work for the default
+    /// syntax set, but may not work for any other syntaxes.
     ///
-    /// In the future I might include a "slow mode" that copies the lines passed in and appends a newline if there isn't one.
-    /// but in the interest of performance currently this hacky fix will have to do.
+    /// In the future I might include a "slow mode" that copies the lines passed in and appends a
+    /// newline if there isn't one, but in the interest of performance currently this hacky fix will
+    /// have to do.
     #[cfg(feature = "yaml-load")]
     pub fn add_from_folder<P: AsRef<Path>>(
         &mut self,
@@ -406,7 +495,7 @@ impl SyntaxSetBuilder {
         Ok(())
     }
 
-    /// Build a `SyntaxSet` from the syntaxes that have been added to this
+    /// Build a [`SyntaxSet`] from the syntaxes that have been added to this
     /// builder.
     ///
     /// ### Linking
@@ -422,9 +511,11 @@ impl SyntaxSetBuilder {
     ///
     /// Linking is done in this build step. So in order to get the best
     /// performance, you should try to avoid calling this too much. Ideally,
-    /// create a `SyntaxSet` once and then use it many times. If you can,
-    /// serialize a `SyntaxSet` for your program and when you run the program,
-    /// directly load the `SyntaxSet`.
+    /// create a [`SyntaxSet`] once and then use it many times. If you can,
+    /// serialize a [`SyntaxSet`] for your program and when you run the program,
+    /// directly load the [`SyntaxSet`].
+    ///
+    /// [`SyntaxSet`]: struct.SyntaxSet.html
     pub fn build(self) -> SyntaxSet {
 
         #[cfg(not(feature = "metadata"))]
@@ -477,6 +568,7 @@ impl SyntaxSetBuilder {
             syntaxes.push(syntax);
         }
 
+        let mut found_more_backref_includes = true;
         for syntax in &syntaxes {
             let mut no_prototype = HashSet::new();
             let prototype = syntax.contexts.get("prototype");
@@ -494,6 +586,38 @@ impl SyntaxSetBuilder {
                     }
                 }
                 Self::link_context(&mut context, syntax, &syntaxes);
+                
+                if context.uses_backrefs {
+                    found_more_backref_includes = true;
+                }
+            }
+        }
+        
+        // We need to recursively mark contexts that include contexts which
+        // use backreferences as using backreferences. In theory we could use
+        // a more efficient method here like doing a toposort or constructing
+        // a representation with reversed edges and then tracing in the
+        // opposite direction, but I benchmarked this and it adds <2% to link
+        // time on the default syntax set, and linking doesn't even happen
+        // when loading from a binary dump.
+        while found_more_backref_includes {
+            found_more_backref_includes = false;
+            // find any contexts which include a context which uses backrefs
+            // and mark those as using backrefs - to support nested includes
+            for context_index in 0..all_contexts.len() {
+                let context = &all_contexts[context_index];
+                if !context.uses_backrefs && context.patterns.iter().any(|pattern| {
+                    match pattern {
+                        Pattern::Include(ContextReference::Direct(id))
+                            if all_contexts[id.index()].uses_backrefs => true,
+                        _ => false,
+                    }
+                }) {
+                    let mut context = &mut all_contexts[context_index];
+                    context.uses_backrefs = true;
+                    // look for contexts including this context
+                    found_more_backref_includes = true;
+                }
             }
         }
 
@@ -644,9 +768,8 @@ impl FirstLineCache {
         let mut regexes = Vec::new();
         for (i, syntax) in syntaxes.iter().enumerate() {
             if let Some(ref reg_str) = syntax.first_line_match {
-                if let Ok(reg) = Regex::new(reg_str) {
-                    regexes.push((reg, i));
-                }
+                let reg = Regex::new(reg_str.into());
+                regexes.push((reg, i));
             }
         }
         FirstLineCache {
@@ -736,6 +859,18 @@ mod tests {
         assert_ops_contain(&ops, &expected);
     }
 
+    #[test]
+    fn can_list_added_syntaxes() {
+        let mut builder = SyntaxSetBuilder::new();
+        builder.add(syntax_a());
+        builder.add(syntax_b());
+        let syntaxes = builder.syntaxes();
+
+        assert_eq!(syntaxes.len(), 2);
+        assert_eq!(syntaxes[0].name, "A");
+        assert_eq!(syntaxes[1].name, "B");
+    }
+
     #[test]
     fn can_add_more_syntaxes_with_builder() {
         let syntax_set_original = {
@@ -770,6 +905,29 @@ mod tests {
         assert_ops_contain(&ops, &expected);
     }
 
+    #[test]
+    fn can_find_unlinked_contexts() {
+        let syntax_set = {
+            let mut builder = SyntaxSetBuilder::new();
+            builder.add(syntax_a());
+            builder.add(syntax_b());
+            builder.build()
+        };
+
+        let unlinked_contexts = syntax_set.find_unlinked_contexts();
+        assert_eq!(unlinked_contexts.len(), 0);
+
+        let syntax_set = {
+            let mut builder = SyntaxSetBuilder::new();
+            builder.add(syntax_a());
+            builder.build()
+        };
+
+        let unlinked_contexts : Vec<String> = syntax_set.find_unlinked_contexts().into_iter().collect();
+        assert_eq!(unlinked_contexts.len(), 1);
+        assert_eq!(unlinked_contexts[0], "Syntax 'A' with scope 'source.a' has unresolved context reference ByScope { scope: <source.b>, sub_context: Some(\"main\") }");
+    }
+
     #[test]
     fn can_use_in_multiple_threads() {
         use rayon::prelude::*;
@@ -819,7 +977,7 @@ mod tests {
             let mut builder = SyntaxSetBuilder::new();
             builder.add(syntax_a());
             builder.add(syntax_b());
-            
+
             let syntax_a2 = SyntaxDefinition::load_from_str(r#"
                 name: A improved
                 scope: source.a
@@ -834,7 +992,7 @@ mod tests {
                 "#, true, None).unwrap();
 
             builder.add(syntax_a2);
-            
+
             let syntax_c = SyntaxDefinition::load_from_str(r#"
                 name: C
                 scope: source.c
@@ -849,7 +1007,7 @@ mod tests {
                 "#, true, None).unwrap();
 
             builder.add(syntax_c);
-            
+
             builder.build()
         };
 
@@ -859,7 +1017,7 @@ mod tests {
         assert_eq!(syntax.name, "A improved");
         syntax = syntax_set.find_syntax_by_first_line(&"syntax a").unwrap();
         assert_eq!(syntax.name, "C");
-        
+
         let mut parse_state = ParseState::new(syntax);
         let ops = parse_state.parse_line("c go_a a", &syntax_set);
         let expected = (7, ScopeStackOp::Push(Scope::new("a2").unwrap()));
